; X-10 Receiver
; Copyright Philip Charles Plunkett    ABACUS ELECTRICS   1996 - 2000

; Client:                    In House
; File Name:                 X10RX129.ASM
; File Date:                 22 June 2000
; Target Device:             PIC 16F84
; Clock Type:                4MHz Xtal, 1uS Instruction Cycle
; Watchdog:                  On
; Code Protection:           Off
; Customer ID Code:          'ABAC'
; Assembler:                 MPASM version  2.30

; Code Revision:             1.28  First released version (16C54) (C) 1996
;                            1.29  First version ported to 16F84  (C) 2000

         TITLE    "X-10 Receiver"

         LIST     P = PIC16F84

; Generic Definitions
W          EQU    H'0000'    ; Result Destination = W Register
F          EQU    H'0001'    ; Result Destination = F Register

; Standard Register Files
INDF       EQU    H'0000'    ; Indirect Addressing Register
RTCC       EQU    H'0001'    ; Counter
PC         EQU    H'0002'    ; Program Counter
STATUS     EQU    H'0003'    ; Status Register
FSR        EQU    H'0004'    ; File Select Register
PORT_A     EQU    H'0005'    ; Port A Input/Output Port
PORT_B     EQU    H'0006'    ; Port B Input/Output Port
EEDATA     EQU    H'0008'    ; EEPROM Data Register
EEADR      EQU    H'0009'    ; EEPROM Address Register
PCLATH     EQU    H'000A'    ; Program Counter upper bits
INTCON     EQU    H'000B'    ; Interrupt Control Register

OPT_REG    EQU    H'0081'    ; Option Register
TRISA      EQU    H'0085'    ; Port A Data Direction Register
TRISB      EQU    H'0086'    ; Port B Data Direction Register
EECON1     EQU    H'0088'    ; EEPROM Control Register 1
EECON2     EQU    H'0089'    ; EEPROM Control Register 1 (not implemented)

; STATUS Register Bits
#define CARRY     STATUS, 0  ; Carry Bit
#define DIGCARRY  STATUS, 1  ; Digit Carry Bit
#define ZERO      STATUS, 2  ; Zero Bit
#define NOT_PD    STATUS, 3  ; Power Down Bit
#define NOT_TO    STATUS, 4  ; Watchdog TimeOut Bit
#define RP0       STATUS, 5  ; Register Page Select - direct
#define RP1       STATUS, 6  ; Register Page Select - direct
#define IRP       STATUS, 7  ; Register Page Select - indirect

; OPTION Register Bits
PS0        EQU    H'0000'    ; Prescaler divide ratio  Eight divide ratios:
PS1        EQU    H'0001'    ; Prescaler divide ratio   WDT  1:1 to 1:128
PS2        EQU    H'0002'    ; Prescaler divide ratio   RTCC 1:2 to 1:256
PSA        EQU    H'0003'    ; Prescaler Assignment   0=RTCC   1=WDT
T0SE       EQU    H'0004'    ; RTCC Signal Edge 0=Lo/Hi 1=Hi/Lo
T0CS       EQU    H'0005'    ; RTCC Signal Source 0=internal 1=RA4
INTEDG     EQU    H'0006'    ; Interrupt Edge Select 0=fall 1=rise
NOT_RBPU   EQU    H'0007'    ; Port B Pull-up Enable

; INTCON Register Bits
RBIF       EQU    H'0000'    ; RB Port Change Interrupt Bit
INTF       EQU    H'0001'    ; RB0/INT Interrupt Flag Bit
T0IF       EQU    H'0002'    ; RTCC Overflow Interrupt Bit
RBIE       EQU    H'0003'    ; RBIF Interrupt Enable
INTE       EQU    H'0004'    ; RB0/INT Interrupt Enable
T0IE       EQU    H'0005'    ; RTIF Interrupt Enable
EEIE       EQU    H'0006'    ; EE Write Complete Interrupt Enable
GIE        EQU    H'0007'    ; Global Interrupt Enable

; EECON1 Register Bits
RD         EQU    H'0000'    ; Read Control Bit
WR         EQU    H'0001'    ; Write Control Bit
WREN       EQU    H'0002'    ; EEPROM Write Enable bit
WRERR      EQU    H'0003'    ; EEPROM Error Flag Bit
EEIF       EQU    H'0004'    ; EEPROM Write Operation Interrupt Flag Bit


; Fuse Definitions and Configurations
_CP_ON     EQU    H'000F'
_CP_OFF    EQU    H'3FFF'

_PWRTE_ON  EQU    H'3FF7'
_PWRTE_OFF EQU    H'3FFF'

_WDT_ON    EQU    H'3FFF'
_WDT_OFF   EQU    H'3FFB'

_LP_OSC    EQU    H'3FFC'    ; 32kHz
_XT_OSC    EQU    H'3FFD'    ; 100kHz - 4MHz
_HS_OSC    EQU    H'3FFE'    ; 4MHz - 20MHz
_RC_OSC    EQU    H'3FFF'    ; 100KhZ - 4MHz

           __FUSES  _CP_OFF&_PWRTE_OFF&_WDT_ON&_XT_OSC

           __MAXRAM H'CF'
           __BADRAM H'07', H'50'-H'7F', H'87'

           __IDLOCS H'ABAC'  ; 2000H - 2003H


; Input/Output Port Assignments

#define TRIAC   PORT_A, 0    ; Triac Gate Drive
#define ZEROX   PORT_A, 1    ; Zero-Crossing Detector Input
#define CARRIER PORT_A, 2    ; Mains Carrier Detect
#define BUTTON  PORT_A, 3    ; Push Button Input

#define UNIT3   PORT_B, 0    ; Unit/Function Code 3
#define UNIT2   PORT_B, 1    ; Unit/Function Code 2
#define UNIT1   PORT_B, 2    ; Unit/Function Code 1
#define UNIT0   PORT_B, 3    ; Unit/Function Code 0
#define HOUSE0  PORT_B, 4    ; House Code 0 \
#define HOUSE1  PORT_B, 5    ; House Code 1 | 0 to 15
#define HOUSE2  PORT_B, 6    ; House Code 2 | re-assigned as
#define HOUSE3  PORT_B, 7    ; House Code 3 / A to O

; Register File Variables
FLAGS    EQU    H'0010'      ; Bits used as status flags
COUNT    EQU    H'0011'      ; Counter used by Wait routines
HOUSE    EQU    H'0012'      ; House code, 0 - 15 == A - P
UNIT     EQU    H'0013'      ; Unit code, 0 - 15 == 1 - 16
RXHOUSE  EQU    H'0014'      ; Received House code, 0 - 15 == A - P
RXUNIT   EQU    H'0015'      ; Received Unit code, 0 - 15 == 1 - 16
RXFUNC   EQU    H'0016'      ; Received Unit code, 0 - 15 == 1 - 16

#define LOAD    FLAGS, 0     ; 0=Load is off, 1=Load is on


; Define Macros
TriacOn  macro
         BCF    TRIAC
                endm

TriacOff macro
         BSF    TRIAC
                endm

GetBit   macro  locn
         CALL   ReadBit
         RLF    locn, F
                endm


;X-10 COMMAND FORMAT
;
; start     unit     house    gap     house        start     function
;      house    start     unit   start     function     house
;
;NB: Total of 94 zero-crossings used, 95 with ND561 (*)
;    Control console/RF remote produce one extra blank half-cycle;
;    (7 instead of 6) in gap:
;
;Control console ND561 via SH624 RF remote; sending A1 ON:
;                                                         (*)
;  1110 01101001 0110100101 1110 01101001 0110100101 0000000>continued
;       0 1 1 0  0 1 1 0 0       0 1 1 0  0 1 1 0 0         >
;  S    A        1          S    A        1          GAP    >
;
;     >1110 01101001 0101100110 1110 01101001 0101100110
;     >     0 1 1 0  0 0 1 0 1       0 1 1 0  0 0 1 0 1
;     >S    A        ON         S    A        ON         DONE
;
;
;Mini-Controller MC563 or Mini-Timer MT522; sending C1 ON:
;
;  1110 01011001 0110100101 1110 01011001 0110100101 000000>
;       0 0 1 0  0 1 1 0 0       0 0 1 0  0 1 1 0 0        >continued
;  S    C        1          S    C        1          GAP   >
;
;     >1110 01011001 0101100110 1110 01011001 0101100110
;     >0 0 1 0  0 0 1 0 1       0 0 1 0  0 0 1 0 1
;     >S    C        ON         S    C        ON         DONE
;

         ORG    00H

; Program starting point

Start
         BSF    RP0          ; Set to page 1 for registers >=$80

         MOVLW  B'00001110'
         MOVWF  TRISA        ; Set Port A as Inputs & Outputs

         MOVLW  B'11111111'
         MOVWF  TRISB        ; Set Port B as Inputs

         MOVLW  B'00000111'  ; RTCC Prescaler, divide by 256
         MOVWF  OPT_REG      ; 256 usec per count (LSB of RTCC)

         BCF    RP0          ; Reset to page 0 for registers <$80

         TriacOff            ; Set Load off
         CLRF   FLAGS
         CLRF   HOUSE
         CLRF   UNIT
         CLRF   RXHOUSE
         CLRF   RXUNIT
         CLRF   COUNT

MainLoop
         CALL   Wait500mSec

Wait1110A
         CLRWDT

         BTFSS  LOAD         ; Continuously re-assert load status
         TriacOff
         BTFSC  LOAD
         TriacOn

         BTFSS  BUTTON       ; Check if manual button pressed,
         GOTO   Manual       ; if so do manual routines

         CALL   ReadBit      ; Wait for Start Sequence 1110
         BTFSS  CARRY        ; 1
         GOTO   Wait1110A
         CALL   ReadBit
         BTFSS  CARRY        ; 1
         GOTO   Wait1110A
         CALL   ReadBit
         BTFSS  CARRY        ; 1
         GOTO   Wait1110A
         CALL   ReadBit
         BTFSC  CARRY        ; 0
         GOTO   Wait1110A

         GetBit RXHOUSE      ; Read first house address
         GetBit RXHOUSE
         GetBit RXHOUSE
         GetBit RXHOUSE
         GetBit RXHOUSE
         GetBit RXHOUSE
         GetBit RXHOUSE
         GetBit RXHOUSE

         GetBit RXUNIT       ; Read first unit address
         GetBit RXUNIT
         GetBit RXUNIT
         GetBit RXUNIT
         GetBit RXUNIT
         GetBit RXUNIT
         GetBit RXUNIT       ; ignore last two bits
         GetBit RXUNIT       ; as always 0 1 == 0

         MOVLW  D'0024'      ; wait 24 zero-crossings, ie
         MOVWF  COUNT        ; to start of 'gap'
BitCount1
         CALL   ReadBit
         DECFSZ COUNT, F
         GOTO   BitCount1

         MOVLW  D'0011'      ; wait a maximum of 10 zero-crossings
         MOVWF  COUNT        ; for next start sequence
Wait1110B
         DECFSZ COUNT, F
         GOTO   W1110B
         GOTO   MainLoop     ; Abort as problem with data sequence
W1110B
         CLRWDT
         CALL   ReadBit      ;     Re-synchronise with pre-function
         BTFSS  CARRY        ; 1   start sequence
         GOTO   Wait1110B
         CALL   ReadBit
         BTFSS  CARRY        ; 1   Can not just count bits as gap
         GOTO   Wait1110B    ;     length varies with transmitter type
         CALL   ReadBit
         BTFSS  CARRY        ; 1
         GOTO   Wait1110B
         CALL   ReadBit
         BTFSC  CARRY        ; 0
         GOTO   Wait1110B

         MOVLW  D'0008'      ; ignore next 8 bits =
         MOVWF  COUNT        ; house address again
BitCount2
         CALL   ReadBit
         DECFSZ COUNT, F
         GOTO   BitCount2

         GetBit RXFUNC       ; Read command function
         GetBit RXFUNC
         GetBit RXFUNC
         GetBit RXFUNC
         GetBit RXFUNC       ; ignore last two bits of function
         GetBit RXFUNC       ; as always 1 0 == 1
         GetBit RXFUNC
         GetBit RXFUNC       ; and ignore rest of data (22 bits)

         CALL   ReadDIPSwitches
         MOVF   HOUSE, W
         SUBWF  RXHOUSE, W
         XORLW  0
         BZ     GoodHouse
         GOTO   MainLoop
GoodHouse
         MOVF   UNIT, W      ; Received House code matches DIP switch
         SUBWF  RXUNIT, W
         XORLW  0
         BZ     GoodUnit
         GOTO   MainLoop
GoodUnit                     ; Received Unit code matches DIP switch
         MOVLW  B'01011001'  ; On function 0010 1
         SUBWF  RXFUNC, W
         XORLW  0
         BZ     PowerOn1
         MOVLW  B'01011010'  ; Off function 0011 1
         SUBWF  RXFUNC, W
         XORLW  0
         BZ     PowerOff1
         GOTO   MainLoop
PowerOn1
         TriacOn
         BSF    LOAD
         GOTO   MainLoop
PowerOff1
         TriacOff
         BCF    LOAD
         GOTO   MainLoop
Manual                       ; Manual operation, de-bounce switch
         CALL   Wait100mSec  ; by reading again after a delay.
         BTFSC  BUTTON       ; Re-check if manual button pressed.
         GOTO   MainLoop
         BTFSS  LOAD
         GOTO   PowerOn2
Poweroff2
         CLRWDT
         TriacOff            ; Manual switch load off
         BCF    LOAD         ; Clear 'load on' flag
         BTFSS  BUTTON       ; Wait here until button released
         GOTO   Poweroff2
         GOTO   MainLoop
PowerOn2
         CLRWDT
         TriacOn             ; Manual switch load on
         BSF    LOAD         ; Set 'load on' flag
         BTFSS  BUTTON       ; Wait here until button released
         GOTO   PowerOn2
         GOTO   MainLoop


ReadBit
         CALL   ZeroCross    ; Call & return after zero crossing
         CALL   Wait250uSec  ; Wait so sample into carrier burst
         BTFSS  CARRIER      ; Check for presence of 120kHz carrier
         GOTO   GB1
         BSF    CARRY        ; Carrier present so set carry flag
         RETLW  0
GB1
         BCF    CARRY        ; Carrier absent so clear carry flag
         RETLW  0


ZeroCross
         BTFSC  ZEROX        ; Check present +ve/-ve status of 50Hz
         GOTO   Z1           ; power then wait for it to change
Z0
         CLRWDT              ; Is low
         BTFSS  ZEROX        ; Loop until goes high
         GOTO   Z0
         RETLW  0
Z1
         CLRWDT              ; Is high
         BTFSC  ZEROX        ; Loop until goes low
         GOTO   Z1
         RETLW  0


Wait100mSec
         MOVLW  D'0006'      ; 6 x 16.384msec = 98.304msec
         MOVWF  COUNT
         GOTO   WaitLoop
Wait500mSec
         MOVLW  D'0030'      ; 30 x 16.384msec = 491.52msec
         MOVWF  COUNT
WaitLoop
         MOVLW  D'0192'      ; 256-64=192,  64 x 256usec = 16.384msec
         MOVWF  RTCC         ; Time delay inner loop
Wait16
         CLRWDT
         MOVF   RTCC, W
         BNZ    Wait16

         DECFSZ COUNT, F
         GOTO   WaitLoop
         RETLW  0


Wait250uSec
         CLRF   RTCC
W0
         CLRWDT              ; Measured at about 273uSecs
         MOVF   RTCC, W
         BZ     W0
         RETLW  0


ReadDIPSwitches
         SWAPF  PORT_B, W    ; Read DIP switch high nibble = house address
         MOVWF  HOUSE        ; and complement for true-logic
         COMF   HOUSE, W     ; Save in HOUSE variable
         ANDLW  H'000F'      ; 0 -> 15 == A -> P
         CALL   HouseTable   ; Look up bit pattern
         MOVWF  HOUSE
         MOVF   PORT_B, W    ; Read DIP switch low nibble = unit address
         ANDLW  H'000F'      ; Unscramble bits and complement for true-logic
         CALL   UnitTable    ; Look up bit pattern & save in UNIT variable
         MOVWF  UNIT         ; 0 -> 15 == 1 -> 16
         RETLW  0
HouseTable
         ADDWF  PC, F        ;       House Code Look-up table:
         RETLW  B'01101001'  ; A = 0110
         RETLW  B'10101001'  ; B = 1110
         RETLW  B'01011001'  ; C = 0010
         RETLW  B'10011001'  ; D = 1010
         RETLW  B'01010110'  ; E = 0001
         RETLW  B'10010110'  ; F = 1001
         RETLW  B'01100110'  ; G = 0101
         RETLW  B'10100110'  ; H = 1101
         RETLW  B'01101010'  ; I = 0111
         RETLW  B'10101010'  ; J = 1111
         RETLW  B'01011010'  ; K = 0011
         RETLW  B'10011010'  ; L = 1011
         RETLW  B'01010101'  ; M = 0000
         RETLW  B'10010101'  ; N = 1000
         RETLW  B'01100101'  ; O = 0100
         RETLW  B'10100101'  ; P = 1100
UnitTable
         ADDWF  PC, F        ;       Unit Code Look-up table:
         RETLW  B'10100101'  ;0>0>F = 16 = 1100 0     Flips bits 0 with 3,
         RETLW  B'10100110'  ;1>8>7 =  8 = 1101 0     and 1 with 2 and
         RETLW  B'10011010'  ;2>4>B = 12 = 1011 0     then complements result.
         RETLW  B'10011001'  ;3>C>3 =  4 = 1010 0     Bits must be twisted
         RETLW  B'10010101'  ;4>2>D = 14 = 1000 0     to undo twisting
         RETLW  B'10010110'  ;5>A>5 =  6 = 1001 0     on PCB layout.
         RETLW  B'10101010'  ;6>6>9 = 10 = 1111 0
         RETLW  B'10101001'  ;7>E>1 =  2 = 1110 0
         RETLW  B'01100101'  ;8>1>E = 15 = 0100 0
         RETLW  B'01100110'  ;9>9>6 =  7 = 0101 0
         RETLW  B'01011010'  ;A>5>A = 11 = 0011 0
         RETLW  B'01011001'  ;B>D>2 =  3 = 0010 0
         RETLW  B'01010101'  ;C>3>C = 13 = 0000 0
         RETLW  B'01010110'  ;D>B>4 =  5 = 0001 0
         RETLW  B'01101010'  ;E>7>8 =  9 = 0111 0
         RETLW  B'01101001'  ;F>F>0 =  1 = 0110 0


         END


